#pragma once

#define MAX_RENDERABLE 20

#include <utils.h>
#include <string>
#include "Renderable.h"
#include "RenderableFactory.h"

/*
* Manages all layers as renderables, and as providers.
*/

class LayerManager
{
public:
	//please call this first.
	void Initialize(GLUI* parentGlui);
	void SetViewportSize(int w, int h);

	bool ShouldUpdateLayers();

	void ResetCoords();
	void IncrementZoom(int delta);
	void ChangeZoom(float zoom);
	void ChangeParameter(float newParameter);
	void IncrementParameter(float delta);
	void MoveCoordinates(int x, int y);
	void SetCoordinates(float x, float y, float zoomX, float zoomY);

	void RemoveAllLayers();

	void UpdateTextures();
	void Render();

	LayerManager(void);
	~LayerManager(void);

	//Layer Management:
	void AddLayer(Renderable * newLayer);
	void RemoveLayer();

	//Renderable* m_baseLayer;
	Renderable* m_renderableList[MAX_RENDERABLE];
	int m_renderableCount;

	//only one - singleton-like
	static LayerManager *m_Manager;
	int m_width, m_height;

private:

	void UpdateWindowMapper();
	void SetSelectedLayer();
	
	//signals the main loop to redraw the viewport.
	bool m_shouldUpdateLayers;

	Renderable *m_currentLayer;
	//int m_selectedLayer;

	int GetIdIndex(int id);

	void SetGuiInformation();
	void SetSelectedVariable();
	void SetInputLayer();
	void RemoveInputLayer();

	void InverseChecked();

	void NavigateInto();
	void NavigateUp();

	static void event_InverseCheck(int value);

	static void event_RemoveLayer(int value);
	static void event_AddLayer(int value);
	static void event_ChangeSelection(int value);
	static void event_SetInputLayer(int value);
	static void event_loadXML(int value);
	static void event_saveXML(int value);

	static void event_VariableChangeSelection(int value);
	static void event_RemoveInputLayer(int value);
	static void event_NavigateInto(int value);
	static void event_NavigateUp(int value);

	void LoadFromXml();
	void SaveToXml();

	GLUI *m_parentGlui;
	GLUI_Panel *m_panelLayers, *m_layerVariables;
	GLUI_Listbox 
#ifdef ENABLE_LAYER_LIST
		*m_lbLayers,
#endif
		*m_lbVariables, *m_lbLayerTypes;
	GLUI_EditText *m_txtVariableValue;

	GLUI_Checkbox *m_cbInverse;

	GLUI_Button
#ifdef ENABLE_LAYER_LIST
		*m_btnRemoveLayer, 
#endif
				*m_btnAddLayer,
				*m_btnSetInputLayer, *m_btnRemoveInputLayer,
				*m_btnNavigateInto, *m_btnNavigateUp,
				*m_btnLoadXML, *m_btnSaveXML;

};