#pragma once

#include <utils.h>
#include "WindowMapping.h"
#include "LayerVariable.h"
#include "RenderableFactory.h"
#include "Serializable.h"

#define MAX_VARIABLES 10

//for now, renderable and gui-able are the same.
class Renderable : public Serializable
{
public:
	Renderable(int TypeID);
	~Renderable(void);

	void Render(float zValue);
	void UpdateTexture();

	virtual void SaveToXml(TiXmlElement* parentNode);
	virtual void LoadFromXml(TiXmlElement* parentNode);

	void MoveCoordinates(int delta_x, int delta_y);
	void UpdateWindowMapper();
	void ResetCoords();
	void ChangeZoom(float zoom);
	void IncrementZoom(float deltazoom);
	void SetCoordinates(float x, float y, float zoomX, float zoomY);

	float GetValueAt(int x, int y);

	void AssignLayerToVariable(int variableIndex, Renderable* subLayer);

	virtual void ChangeParameter(int parameterIndex, float newParameter);
	virtual void IncrementParameter(int parameterIndex, float deltaParameter);

	void Initialize();

	Renderable *ParentLayer;
	int m_variableCount;
	LayerVariable m_layerVariables[MAX_VARIABLES];
	int id;
	int m_typeID;

	bool m_inverse;


protected:
	virtual void SetVariables() = 0;
	void WriteBitmap();

	virtual float GetRawValueAt(int x, int y) = 0;

	//bitmap dimensions
	int m_width, m_height;

	POINT_FLOAT m_coords;
	float m_zoomx, m_zoomy;

	WindowMapping m_windowMapping;
	LayerManager* m_layerManager;

	//physical bitmap
	GLubyte* m_bitmapBits;

	//GL texture!
	GLuint m_Texture;

private:
	static int lastID;
};