
#include "TinyXml.h"
#include <string>
using namespace std;
#include "FilePromptUtil.h"

#include "Serializable.h"
#include "LayerManager.h"

Serializable::Serializable(void)
{
}

Serializable::~Serializable(void)
{
}


void LayerManager::LoadFromXml()
{
	string path = FilePromptUtil::LoadFileName();
	if(path.length() == 0)
		return;

	TiXmlDocument doc( path.c_str());
	doc.LoadFile();

	if(doc.Error())
		return;
	
	TiXmlElement * root = doc.FirstChildElement( "NoiseViewer" );
	if(!root) return;

	TiXmlElement * layers = root->FirstChildElement( "Layers" );
	if(!layers) return;

	TiXmlElement *child = 0;
	while( child = (TiXmlElement*)layers->IterateChildren( child ) )
	{
		//any child here is actually a layer.
		int type_id;
		child->QueryIntAttribute("TypeID", &type_id);
		Renderable* newLayer = RenderableFactory::CreateRenderable(type_id);
		this->AddLayer(newLayer);
		newLayer->LoadFromXml(child);
	}
}

void LayerManager::SaveToXml()
{
	string path = FilePromptUtil::SaveFileName();
	if(path.length() == 0)
		return;
	
	TiXmlDocument doc;
	TiXmlDeclaration* decl = new TiXmlDeclaration( "1.0", "", "" );
	doc.LinkEndChild( decl );  

	TiXmlElement * root = new TiXmlElement( "NoiseViewer" );
	doc.LinkEndChild( root );  

	TiXmlElement * layers = new TiXmlElement( "Layers" );  
	root->LinkEndChild( layers );

	for(int i = 0; i < m_renderableCount; i++)
	{
		m_renderableList[i]->SaveToXml(layers);
	}

	//ensure adding ".xml" to end!
	int len = path.length();
	string end = path.substr(len - 4, 4);
	if(end.compare(".xml") != 0)
		path.append(".xml");
	doc.SaveFile(path.c_str());
}

void Renderable::LoadFromXml(TiXmlElement* parentNode)
{
	parentNode->QueryFloatAttribute("x", &m_coords.x);
	parentNode->QueryFloatAttribute("y", &m_coords.y);
	parentNode->QueryFloatAttribute("zoomx", &m_zoomx);
	parentNode->QueryFloatAttribute("zoomy", &m_zoomy);
	parentNode->QueryIntAttribute("inverse", (int*) &m_inverse);

	TiXmlElement * variables = parentNode->FirstChildElement( "Variables" );
	if(!variables) return;

	TiXmlElement *child = 0;
	int runningIndex = 0;
	while( child = (TiXmlElement*)variables->IterateChildren( child ) )
	{
		//any child here is actually a variable.
		if(runningIndex >= m_variableCount)
			return; //you have an older version of noiseviewer than this xml.

		m_layerVariables[runningIndex].LoadFromXml(child);
	}
}

void Renderable::SaveToXml(TiXmlElement* parentNode)
{
	TiXmlElement *layer = new TiXmlElement("Layer");
	parentNode->LinkEndChild(layer);

	layer->SetAttribute("TypeID", m_typeID);
	
	layer->SetDoubleAttribute("x", m_coords.x);
	layer->SetDoubleAttribute("y", m_coords.y);
	layer->SetDoubleAttribute("zoomx", m_zoomx);
	layer->SetDoubleAttribute("zoomy", m_zoomy);
	layer->SetAttribute("inverse", (int)m_inverse);

	TiXmlElement *variables = new TiXmlElement("Variables");
	layer->LinkEndChild(variables);
	for(int i = 0; i < m_variableCount; i++)
	{
		m_layerVariables[i].SaveToXml(variables);
	}

}

void LayerVariable::SaveToXml(TiXmlElement *parentElement)
{
	TiXmlElement *variable = new TiXmlElement("Variable");
	parentElement->LinkEndChild(variable);
	variable->SetAttribute("name", name);
	variable->SetDoubleAttribute("MaxValue", m_maxValue);
	variable->SetDoubleAttribute("MinValue", m_minValue);
	variable->SetDoubleAttribute("OverrideValue", m_overrideValue);

	//put renderable here:
	if(renderable_value != NULL)
		renderable_value->SaveToXml(variable);

}

void LayerVariable::LoadFromXml(TiXmlElement *parentElement)
{
	parentElement->QueryFloatAttribute("MaxValue", &m_maxValue);
	parentElement->QueryFloatAttribute("MinValue", &m_minValue);
	parentElement->QueryFloatAttribute("OverrideValue", &m_overrideValue);

	//put renderable here:
	TiXmlElement *renderableEl = parentElement->FirstChildElement("Layer");
	if(renderableEl != NULL)
	{
		int type_id;
		renderableEl->QueryIntAttribute("TypeID", &type_id);
		Renderable* newInput = RenderableFactory::CreateRenderable(type_id);
		newInput->LoadFromXml(renderableEl);


		this->SetRenderable(newInput);
	}
}